// --------------------------------------------------------------- //
// Ken Long's Super NuCube!										//
// 																//
// NuCube.c														//
// 																//
// "Super" version made super by Mark Hanrek, by adding trick Mac  //
// standard stuff - even if only shell form - and adding GWorld    //
// source.															//
// 																//
// --------------------------------------------------------------- //

// ---------------------- D E F I N I T I O N S ------------------ //


#define	rWIND			128		// window resource

#define	kCubeSide		160		// pixels per side

#define kHorizOffset	20		// pixels in from window's left
#define kVertOffset		15		// pixels down from window's top



// ------------------------- I N C L U D E S --------------------- //

#include "NuCube.h"



// ------------------------- G L O B A L S --------------------- //


Boolean			gQuitApplication;	// Setting this to true allows a clean exit
Boolean			gInBackground;

WindowPtr		gAppWindow;			// Windows and GWorlds are the same thing, so
GWorldPtr		gOffscreenGWorld;	// use only Get/SetGWorld calls in this app.

Rect			gProjectionRect;	// The "projection screen" area in the window
Rect			gOffscreenRect;		// The offscreen rect's upper-left is 0,0

ControlHandle	gButtonOne;			// An initial button for starters


// -------------------- P R O T O T Y P E S --------------------- //

void 		InitializeApplication( void );
void 		SetupWindow( void );

void		EventLoop( void );
void 		DoEvent( EventRecord *event );
void 		DoMouseDown( EventRecord *event );
void 		DoContentClick( WindowPtr window, EventRecord *event );
void 		DoUpdate( EventRecord *event );

void 		RotateCubeOnce( void );
GWorldPtr 	CreateOffscreenGWorld( Rect *theRect );
void 		MoveAlongHome( void );



////


//*******************************************************************//

void main( void ) 
{
	InitializeApplication();
	SetupWindow();
	EventLoop();
	MoveAlongHome();
}

//*******************************************************************//

void InitializeApplication( void ) 
{
	InitGraf(&thePort);
	InitFonts();
	InitWindows();
	InitMenus();	
	TEInit();
	InitDialogs((ProcPtr) nil );
	InitCursor();
	
	gQuitApplication = false;
	gInBackground = false;
}

//*******************************************************************//
//  Create window, offscreen GWorld, and one control for starters	 //
//*******************************************************************//

void SetupWindow( void ) 
{
	Rect	tempRect;
	
	//------ Set up our main window, man

	gAppWindow = GetNewWindow( rWIND, nil, (WindowPtr) -1L);
	
	SetPort( gAppWindow );
	ShowWindow( gAppWindow );

	SetRect( &gProjectionRect, kHorizOffset, kVertOffset,  kHorizOffset+kCubeSide, kVertOffset+kCubeSide );


	//------ Set up an offscreen GWorld for preparing the next cube's image
		
	SetRect( &gOffscreenRect, 0, 0, kCubeSide, kCubeSide );
	
	gOffscreenGWorld = CreateOffscreenGWorld( &gOffscreenRect );
	
	
	//------ Create a button, just to get you started!
	
	SetRect( &tempRect, 300, 20, 390, 40 ); // always make buttons 20 pixels high
	
	gButtonOne = NewControl( gAppWindow, &tempRect, "\pWarp Drive", true, 0,0,0, pushButProc, 0L );
}

//*******************************************************************//

void EventLoop( void )
{
	EventRecord		event;

	while (!gQuitApplication) 
	{
		if (!WaitNextEvent( everyEvent, &event, 0, 0 ))
			event.what = nullEvent;

		DoEvent( &event );
	}
}

//*******************************************************************//

void DoEvent( EventRecord *event )
{
	switch(event->what) {

		case nullEvent:			
			// DoIdleTasks(event);
			break;

		case mouseDown:			
			DoMouseDown(event);
		break;

		case autoKey:
		case keyDown:			
			// DoKeyDown(event);
		break;

		case activateEvt:		
			// DoActivate(event);
		break;

		case updateEvt:			
			DoUpdate(event);
		break;

//		case kHighLevelEvent:	
//			AEProcessAppleEvent(event);
//		break;

		case diskEvt:			
			// DoDisketteInsertEvent();
			break;
			
		case osEvt:
			switch (( event->message >> 24) & 0xFF) {
			
				case mouseMovedMessage:
					break;

				case suspendResumeMessage:
					gInBackground = !((event->message) & resumeFlag);
					if ( !gInBackground )
						; //DoActivate(event); 		// Moving to foreground
					else
						; //DoDeactivate(evenet);	// Moving to background
					break;
			}
			break;
	}
	
	//----- Rotate the cube once each time through the event loop
	
	RotateCubeOnce();
}

//*******************************************************************//

void DoMouseDown( EventRecord *event )
{
	WindowPtr	window;
	short		part;
	
	
	part = FindWindow( event->where, &window );

	switch(part) {

		case inContent:
			DoContentClick( window, event );
			break;

		case inDrag:
			DragWindow( window, event->where, &qd.screenBits.bounds);
			break;	

		case inGoAway:
			if ( TrackGoAway( window, event->where ))
				gQuitApplication = true;
			break;

		case inGrow:
			// DoGrowWindow( window, event );
			break;

		case inMenuBar:	
			// DoAdjustMenus();
			// DoMenuCommand( MenuSelect( event->where ));
			break;

		case inSysWindow:	// Let the system handle the mouseDown.
			SystemClick(event, window);
			break;

		case inZoomIn:
		case inZoomOut:
			// DoZoomWindow( window, event, part);
			break;
	}
}

//*******************************************************************//

void DoContentClick( WindowPtr window, EventRecord *event )
{
	ControlHandle	theControl;
	short			part;
	Point			clickLoc;
	
	clickLoc = event->where;
	GlobalToLocal( &clickLoc );
	
	if ( part = FindControl( clickLoc, window, &theControl ))
	{
		if ( TrackControl( theControl, clickLoc, nil ))
		{
			SysBeep(1);
	
		}
	}
}

//*******************************************************************//

void DoUpdate( EventRecord *event )
{
	WindowPtr	window;
	
	window = (WindowPtr) event->message;
	
	BeginUpdate( window );
	DrawControls( window );
	EndUpdate( window );
}

//*******************************************************************//

void RotateCubeOnce( void )
{
	long			ticksNow;
	GWorldPtr		windowGW;
	GDHandle		windowGD;
	PixMapHandle	thePixMap;

	GetGWorld( &windowGW, &windowGD ); 	// Save whatever we had before

	SetGWorld( gOffscreenGWorld, nil );

	if ( LockPixels( thePixMap = GetGWorldPixMap( gOffscreenGWorld )))
	{
		EraseRect(&gOffscreenGWorld->portRect);

		//=========================================================================

		fi1 = (fi1 + 7) m2PI;
		fi2 = (fi2 + 1) m2PI;

		b = sinus[fi2];
		c = (sinus[( 64 - fi2) m2PI] * 91) / 128; 	/* 91/128 = sqrt(2) */

		p0v = 85 +  sinus[fi1];
		p0h = 85 +  (b * sinus[( 64 - fi1) m2PI]) / 64;

		p1v = 85 +  sinus[(64 + fi1) m2PI];
		p1h = 85 +  (b * sinus[( - fi1) m2PI]) / 64;

		p2v = 85 +  sinus[(128 + fi1) m2PI];
		p2h = 85 +  (b * sinus[(- 64 - fi1) m2PI]) / 64;

		p3v = 85 +  sinus[(192 + fi1) m2PI];
		p3h = 85 +  (b * sinus[(- 128 - fi1) m2PI]) / 64;

		//-----------------------------------------------------------------
		if( ! ( ((((fi2 - 192)m2PI) < 64) && (((fi1 -  32)m2PI) < 128)) ||
				((((fi2 - 128)m2PI) < 64) && (((fi1 - 160)m2PI) < 128)) ) )
		{
			MoveTo(p0h - c, p0v);
			LineTo(p1h - c, p1v);
		}
		if( ! ( ((((fi2 - 192)m2PI) < 64) && (((fi1 - 224)m2PI) < 128)) ||
				((((fi2 - 128)m2PI) < 64) && (((fi1 -  96)m2PI) < 128)) ) )
		{
			MoveTo(p1h - c, p1v); 
			LineTo(p2h - c, p2v);
		}
		if( ! ( ((((fi2 - 192)m2PI) < 64) && (((fi1 - 160)m2PI) < 128)) ||
				((((fi2 - 128)m2PI) < 64) && (((fi1 -  32)m2PI) < 128)) ) )
		{
			MoveTo(p2h - c, p2v); 
			LineTo(p3h - c, p3v);
		}
		if( ! ( ((((fi2 - 192)m2PI) < 64) && (((fi1 -  96)m2PI) < 128)) ||
				((((fi2 - 128)m2PI) < 64) && (((fi1 - 224)m2PI) < 128)) ) )
		{
			MoveTo(p3h - c, p3v); 
			LineTo(p0h - c, p0v);
		}
		//-----------------------------------------------------------------
		if( ! ( ((((fi2)m2PI) < 64) 	 && (((fi1 -  32)m2PI) < 128)) ||
				((((fi2 - 64)m2PI) < 64) && (((fi1 - 160)m2PI) < 128)) ) )
		{
			MoveTo(p0h + c, p0v); 
			LineTo(p1h + c, p1v);
		}
		if( ! ( ((((fi2)m2PI) < 64) 	  && (((fi1 - 224)m2PI) < 128)) ||
				((((fi2 -  64)m2PI) < 64) && (((fi1 -  96)m2PI) < 128)) ) )
		{
			MoveTo(p1h + c, p1v);
			LineTo(p2h + c, p2v);
		}
		if( ! ( ((((fi2)m2PI) < 64) && (((fi1 - 160)m2PI) < 128)) ||
				((((fi2 -  64)m2PI) < 64) && (((fi1 -  32)m2PI) < 128)) ) )
		{
			MoveTo(p2h + c, p2v);
			LineTo(p3h + c, p3v);
		}
		if( ! ( ((((fi2	  )m2PI) < 64) && (((fi1 -  96)m2PI) < 128)) ||
				((((fi2 -  64)m2PI) < 64) && (((fi1 - 224)m2PI) < 128)) ) )
		{
			MoveTo(p3h + c, p3v);
			LineTo(p0h + c, p0v);
		}
		//-----------------------------------------------------------------
		if( ! ( ((((fi2 -  64)m2PI) < 128) && (((fi1 - 224)m2PI) < 64)) ||
				((((fi2 - 192)m2PI) < 128) && (((fi1 -  96)m2PI) < 64)) ) )
		{
			MoveTo(p0h + c, p0v);
			Line(-2 * c, 0);
		}
		if( ! ( ((((fi2 -  64)m2PI) < 128) && (((fi1 - 160)m2PI) < 64)) ||
				((((fi2 - 192)m2PI) < 128) && (((fi1 -  32)m2PI) < 64)) ) )
		{
			MoveTo(p1h + c, p1v);
			Line(-2 * c, 0);
		}
		if( ! ( ((((fi2 -  64)m2PI) < 128) && (((fi1 -  96)m2PI) < 64)) ||
				((((fi2 - 192)m2PI) < 128) && (((fi1 - 224)m2PI) < 64)) ) )
		{
			MoveTo(p2h + c, p2v);
			Line(-2 * c, 0);
		}
		if( ! ( ((((fi2 -  64)m2PI) < 128) && (((fi1 -  32)m2PI) < 64)) ||
				((((fi2 - 192)m2PI) < 128) && (((fi1 - 160)m2PI) < 64)) ) )
		{
			MoveTo(p3h + c, p3v);
			Line(-2 * c, 0);
		}
		
		//=========================================================================
		//
		//  Always set the current port to a window's port before drawing from
		//  a GWorld, so that the Window Manager can "clip" the image properly
		//  should there be other windows in front. Comment out the next statement
		//  and notice what happens when you drag another window in front of 
		//  the NuCube window.  :)
		//
		
		SetGWorld( windowGW, windowGD ); 	// Focus on a real window

		CopyBits((BitMapPtr)*thePixMap, (BitMapPtr)&gAppWindow->portBits, 
					&gOffscreenRect, &gProjectionRect, srcCopy, nil );

		UnlockPixels( thePixMap );
	}
	
	SetGWorld( windowGW, windowGD ); 	// Okay if this is redundant
}


//*******************************************************************//
//  Use the following function for the rest of your programming 	 //
//	days. 															 //
//	Do not alter it in any way. It is a complete functional unit 	 //
//	doing all REQUIRED things.										 //
//*******************************************************************//

GWorldPtr CreateOffscreenGWorld( Rect *theRect )
{
	CGrafPtr    currentPort;
	GDHandle    currentGDevice;
	GWorldPtr   offScreen;
	QDErr       result;

	GetGWorld (&currentPort, &currentGDevice);
	
	//-- If there is not enough application memory, try MultiFinder 
	//-- temporary memory.
	
	if (( result = NewGWorld( &offScreen, 0, theRect, nil, nil, 0L )) != noErr)
		if (( result = NewGWorld( &offScreen, 0, theRect, nil, nil, useTempMem)) != noErr)
			return ( nil );
			
	SetGWorld( offScreen, nil );
	
	//-- Initialize the clipping "hole" to be the same as the 
	//-- drawing area.

	ClipRect(&offScreen->portRect);	

	//-- The GWorld has random stuff in it, so we'll erase it to 
	//-- white.
	
	if ( LockPixels( GetGWorldPixMap( offScreen )))
	{
		ForeColor( blackColor );
		BackColor( whiteColor );
		EraseRect(&offScreen->portRect);
		UnlockPixels(GetGWorldPixMap (offScreen));
	}

	SetGWorld(currentPort, currentGDevice);
	return offScreen;
}



//*******************************************************************//
// Shut down with charm, grace and decorum.							 //
//*******************************************************************//

void MoveAlongHome( void ) 
{
	DisposeWindow( gAppWindow );		// Not "CloseWindow"
	DisposeGWorld( gOffscreenGWorld ); 
}


